<?php namespace BoostMyShop\Agent\Helper;

Memory::setMemoryLimit();

use \Magento\Framework\UrlInterface;

/**
 * @copyright 2015-2016 BoostMyShop (http://www.boostmyshop.com)
 * @license   http://opensource.org/licenses/osl-3.0.php  Open Software License (OSL 3.0)
 */
class Catalog extends \Magento\Framework\App\Helper\AbstractHelper
{
    const FILENAME = 'BoostmyshopCatalog.xml';
    /** @var string */
    protected $filePath;
    /** @var array */
    protected $configuration;
    /** @var \DOMDocument */
    protected $dom;
    /** @var \Magento\Store\Model\StoreManagerInterface */
    protected $_storeManager;
    /** @var \BoostMyShop\Agent\Helper\Catalog\Products */
    protected $_productsHelper;
    /** @var \Magento\Catalog\Model\ResourceModel\Product */
    protected $_resourceProduct;
    /** @var  \Magento\Catalog\Model\Product\Attribute\Repository */
    protected $_attributeRepository;
    /** @var \Magento\Framework\Filesystem\DirectoryList */
    protected $_directoryList;
    /** @var \Magento\CatalogInventory\Api\StockRegistryInterface */
    protected $_stockRegistry;

    public function __construct(
        \BoostMyShop\Agent\Helper\Catalog\Products $productsHelper,
        \Magento\Framework\Filesystem\DirectoryList $directoryList,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Catalog\Model\ResourceModel\Product $resourceProduct,
        \Magento\Catalog\Model\Product\Attribute\Repository $attributeRepository,
        \Magento\CatalogInventory\Api\StockRegistryInterface $stockRegistry,
        \Magento\Framework\App\Helper\Context $context
    )
    {
        parent::__construct($context);
        $this->_storeManager = $storeManager;
        $this->_productsHelper = $productsHelper;
        $this->_directoryList = $directoryList;
        $this->_stockRegistry = $stockRegistry;
        $this->_resourceProduct = $resourceProduct;
        $this->_attributeRepository = $attributeRepository;

        $this->filePath = $this->_directoryList->getPath(\Magento\Framework\App\Filesystem\DirectoryList::VAR_DIR) . '/' . self::FILENAME;
    }

    public function generate($configuration)
    {
        $this->_logger->info('START CATALOG GENERATION');
        $this->configuration = $configuration;

        if (file_exists($this->filePath) && (time() - filemtime($this->filePath) < 72000)) {
            return $this->filePath;
        }

        $this->deletePreviousFile();
        $this->dom = new \DOMDocument('1.0', 'UTF-8');
        $this->dom->formatOutput = true;
        $this->addProducts();
        $this->dom->save($this->filePath);

        return $this->filePath;
    }

    private function deletePreviousFile()
    {
        if (file_exists($this->filePath)) {
            unlink($this->filePath);
        }

        return $this;
    }

    private function addProducts()
    {
        $catalogNode = $this->dom->createElement('catalog');
        $this->dom->appendChild($catalogNode);

        $productsNode = $this->dom->createElement('products');
        $catalogNode->appendChild($productsNode);

        $products = $this->_productsHelper->getCollection($this->configuration['storeId'], $this->configuration['attributes']);
        foreach ($products as $product) {
            $this->addProduct($productsNode, $product);
        }
    }

    private function addProduct(\DomElement $productsNode, \Magento\Catalog\Model\Product $product)
    {
        $productNode = $this->dom->createElement('product');
        $productsNode->appendChild($productNode);
        $this->addProductData($productNode, $product);
    }

    private function addProductData(\DOMElement $productNode, \Magento\Catalog\Model\Product $product)
    {
        $this->addProductAttributes($productNode, $product);
        $this->addCategoriesAndWebsites($productNode, $product);
        $this->addImageUrl($productNode, $product);
        $this->addStockItemData($productNode, $product);
    }

    private function addProductAttributes(\DOMElement $productNode, \Magento\Catalog\Model\Product $product)
    {
        foreach ($product->getData() as $attributeCode => $attributeValue) {

            if(is_array($attributeValue)) {
                continue;
            }

            if(in_array($attributeCode, $this->configuration['attributes'])) {
                $attribute = $this->_attributeRepository->get($attributeCode);
                if(in_array($attribute->getFrontendInput(), ['select', 'multiselect'])) {
                    $attributeValue = $this->getValueForListAttribute($attribute, $attributeValue);
                    if (is_array($attributeValue)) {
                        $attributeNode = $this->dom->createElement($attributeCode);
                        $attributeNode->setAttribute('type', 'array');
                        foreach ($attributeValue as $value) {
                            $attributeNode->appendChild($this->dom->createElement('value', $value));
                        }
                        $productNode->appendChild($attributeNode);

                        continue;
                    }
                }
            }

            if ($attributeCode == 'sku') {
                $attributeCode = 'reference';
            }

            if($attributeCode == 'entity_id') {
                $attributeCode = 'sku';
            }

            if ($attributeCode == 'created_at' || $attributeCode == 'updated_at') {
                $attributeValue = (new \DateTime($attributeValue))->format('c');
            }

            $attributeNode = $this->dom->createElement($attributeCode);
            $this->dom->appendChild($attributeNode);
            $attributeNode->appendChild($this->dom->createCDATASection($attributeValue));
            $productNode->appendChild($attributeNode);
        }
    }

    private function getValueForListAttribute($attribute, $attributeValue)
    {
        return $attribute->getFrontEnd()->getOption($attributeValue);
    }

    private function addCategoriesAndWebsites(\DOMElement $productNode, \Magento\Catalog\Model\Product $product)
    {
        $categoryNode = $this->dom->createElement('categories');
        $categoryNode->setAttribute('type', 'array');
        foreach ($this->getCategoryCollection($product) as $category) {
            $categoryNode->appendChild($this->dom->createElement('value', htmlspecialchars($category->getName())));
        }
        $productNode->appendChild($categoryNode);


        $websiteNode = $this->dom->createElement('websites');
        $websiteNode->setAttribute('type', 'array');
        foreach ($product->getWebsiteIds() as $websiteId) {
            $websiteNode->appendChild($this->dom->createElement('value', $websiteId));
        }
        $productNode->appendChild($websiteNode);
    }

    private function getCategoryCollection(\Magento\Catalog\Model\Product $product)
    {
        return $this->_resourceProduct->getCategoryCollection($product)->addAttributeToSelect(array('name'));
    }

    private function addImageUrl(\DOMElement $productNode, \Magento\Catalog\Model\Product $product)
    {
        $imageNode = $this->dom->createElement('image_url');
        $mediaUrl = $this->_storeManager->getDefaultStoreView()->getBaseUrl(UrlInterface::URL_TYPE_MEDIA);
        $imageNode->appendChild($this->dom->createCDATASection($mediaUrl . 'catalog/product' . $product->getThumbnail()));
        $productNode->appendChild($imageNode);
    }

    private function addStockItemData(\DOMElement $productNode, \Magento\Catalog\Model\Product $product)
    {
        $stockItem = $this->_stockRegistry->getStockItem($product->getId(), $product->getStore()->getWebsiteId());
        $stockItemNode = $this->dom->createElement('stock_item');
        $stockItemNode->appendChild($this->dom->createElement('qty', $stockItem->getQty()));
        $stockItemNode->appendChild($this->dom->createElement('is_in_stock', $stockItem->getIsInStock()));
        $stockItemNode->appendChild($this->dom->createElement('manage_stock', $stockItem->getManageStock()));
        $productNode->appendChild($stockItemNode);
    }
}
